function [cc,Iter_AE,p_hat,rank] = DSFLocUknPropSpdStnySrc_AE(s,s_dot,f,Qe,f0)
% function [cc,Iter_AE,p_hat,rank] = DSFLocUknPropSpdStnySrc_AE(s,s_dot,f,Qe,f0)
%
% AE method for the stationary source localization by frequency
% measureemnts when signal propagation speed is not known.
% 
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   Qe:       (N x N), the covariance matrix of measurement noise
%   f0:       (1 x 1), the carrier frequency
%
% Output:
%   cc:       (1 x 1), the estimate of signal propagation speed
%   Iter_AE:  (1 x 1), the iteration times of AE method
%   p_hat:    (2 x 1), the estimate of source position
%   rank:     (1 x 1), the rank of solution of AE-subproblem1 in the last iteration
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency
% measurements in unknown signal propagation speed environments," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	Nov. 1, 2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f); %number of sensors
c0 = 1.5; %initial signal propagation speed (km/s)
% ============================step 1=======================================
Iter_AE = 0;%to count the number of iteration times
cc = 0;
W = inv(Qe); %fix the weighting matrix to Q^-1 in step 1

while norm(c0-cc)>1e-2&&Iter_AE<50
    cc = c0;   
    [p_hat,rank] = DSFLocUknPropSpdStnySrc_AESubPrbm1(s,s_dot,f,f0,c0,W);%solution of AE-subproblem1
    for i=1:N
        r(i) = (f(i)-f0)/f0;
        a(i) = r(i)^2*norm(p_hat-s(:,i))^2;
        g(i) = ((p_hat-s(:,i))'*s_dot(:,i))^2;
    end
    c0 = sqrt(inv(a*W*a')*a*W*g');
    Iter_AE=Iter_AE+1;
end
% ============================step 2=======================================
Iter = 0;
cc = 1.5;%initial signal propagation speed (km/s)
while norm(c0-cc)>1e-5&&Iter<50
    cc = c0;
    
    B = [];
    for j=1:N
        B = blkdiag(B,2*r(j)*c0^2/f0*norm(p_hat-s(:,j))^2);
    end
    W = inv(B'*Qe*B);
    
    [p_hat,rank] = DSFLocUknPropSpdStnySrc_AESubPrbm1(s,s_dot,f,f0,c0,W);%solution of AE-subproblem1
    for i=1:N
        r(i) = (f(i)-f0)/f0;
        a(i) = r(i)^2*norm(p_hat-s(:,i))^2;
        g(i) = ((p_hat-s(:,i))'*s_dot(:,i))^2;
    end
    c0 = sqrt(inv(a*W*a')*a*W*g');
    Iter = Iter+1;
end

Iter_AE = Iter_AE+Iter;
cc = c0;

end


% #########################################################################
% #########################################################################
function [p,rank] = DSFLocUknPropSpdStnySrc_AESubPrbm1(s,s_dot,f,f0,c,W)
% function [p,rank] = DSFLocUknPropSpdStnySrc_AESubPrbm1(s,s_dot,f,f0,c,W)
%
% AE-subproblem1 for stationary source localization using frequency
% measurements when signal propagation speed is not known.
%
% Input:
%   s:        (2 x N), the true value of sensor positions, N = number of sensors
%   s_dot:    (2 x N), the true value of sensor velocities
%   f:        (1 x N), the frequency measurements
%   f0:       (1 x 1), the carrier frequency
%   c:        (1 x 1), signal propagation speed
%   W:        (N x N), weighting matrx
%
% Output:
%   p:       (2 x 1), the estimate of source position
%   rank:    (1 x 1), the rank of SDP solution
%
% Reference:
% D. Lin, G. Wang, K. C. Ho, and L. Huang, "Source localization by frequency
% measurements in unknown signal propagation speed environments," IEEE
% Trans. Aerosp. Electron. Syst., vol. 59, no. 4, pp. 3953-3970, Aug. 2023.
%
% Danyan Lin, Gang Wang, K. C. Ho	2023
%
%       Copyright (C) 2023
%       the Faculty of Electrical Engineering and Computer Science
%       Ningbo University
%       Ningbo 315211, China
%       wanggang@nbu.edu.cn
%

warning('off');

N = length(f);

b = [];
d = [];

for i=1:N
    d(i) = -c*(f0-f(i))/f0;
    A(i,:) = [2*s(:,i)'*s_dot(:,i)*s_dot(:,i)'-2*d(i)^2*s(:,i)',-s_dot(1,i)^2,-2*s_dot(1,i)*s_dot(2,i),-s_dot(2,i)^2,d(i)^2];
    b = [b;(s(:,i)'*s_dot(:,i))^2-d(i)^2*norm(s(:,i))^2];
end

W = W/trace(W);
cvx_clear
cvx_begin sdp
cvx_solver sdpt3
cvx_quiet(1)
cvx_precision best
variable Y(6,6) symmetric
variable y(6,1)

minimize ((trace(A'*W*A*Y)-2*b'*W*A*y+b'*W*b)*1e10)

subject to
[Y y;y' 1]>=0;
y(3) == Y(1,1);
y(4) == Y(1,2);
y(5) == Y(2,2);
y(6) == y(3)+y(5);

Y(1,4) == Y(2,3);
Y(1,5) == Y(2,4);
Y(1,6) == Y(1,3)+Y(2,4);
Y(2,6) == Y(1,4)+Y(2,5);
Y(3,6) == Y(3,3)+Y(4,4);
Y(4,6) == Y(3,4)+Y(4,5);
Y(5,6) == Y(4,4)+Y(5,5);
Y(6,6) == Y(3,3)+2*Y(4,4)+Y(5,5);

cvx_end

p = y(1:2);

rank=0;
lambda=[];
lambda=eig(Y);
if lambda(6)/lambda(5)>=1e5
    rank=1;
end

p = y(1:2);

end



